@@ -347,7 +347,7 @@ atk_object_class_init (AtkObjectClass *klass)
   atk_object_signals[PROPERTY_CHANGE] =
     g_signal_new ("property_change",
                   G_TYPE_FROM_CLASS (klass),
-                  G_SIGNAL_RUN_LAST,
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   G_STRUCT_OFFSET (AtkObjectClass, property_change),
                   (GSignalAccumulator) NULL, NULL,
                   g_cclosure_marshal_VOID__POINTER,
@@ -629,10 +629,10 @@ atk_object_set_name (AtkObject    *accessible,
 
   klass = ATK_OBJECT_GET_CLASS (accessible);
   if (klass->set_name)
-  {
-    (klass->set_name) (accessible, name);
-    g_object_notify (G_OBJECT (accessible), atk_object_name_property_name);
-  }
+    {
+      (klass->set_name) (accessible, name);
+      g_object_notify (G_OBJECT (accessible), atk_object_name_property_name);
+    }
 }
 
 /**
@@ -653,10 +653,10 @@ atk_object_set_description (AtkObject   *accessible,
 
   klass = ATK_OBJECT_GET_CLASS (accessible);
   if (klass->set_description)
-  {
-    (klass->set_description) (accessible, description);
-    g_object_notify (G_OBJECT (accessible), atk_object_name_property_description);
-  }
+    {
+      (klass->set_description) (accessible, description);
+      g_object_notify (G_OBJECT (accessible), atk_object_name_property_description);
+    }
 }
 
 /**
@@ -745,6 +745,40 @@ atk_object_remove_property_change_handler  (AtkObject *accessible,
     (klass->remove_property_change_handler) (accessible, handler_id);
 }
 
+/**
+ * atk_object_notify_state_change:
+ * @accessible: an #AtkObject
+ * @state: an #AtkState whose state is changed
+ * @value : a gboolean which indicates whether the state is being set on or off
+ * 
+ * Emits a property_change signal for the specified state. 
+ * This signal is caught by an #AtkPropertyChangeHandler
+ **/
+void
+atk_object_notify_state_change (AtkObject *accessible,
+                                AtkState  state,
+                                gboolean  value)
+{
+  AtkPropertyValues  values;
+
+  memset (&values.old_value, 0, sizeof (GValue));
+  memset (&values.new_value, 0, sizeof (GValue));
+  values.property_name = atk_object_name_property_state;
+  if (value)
+    {
+       g_value_init (&values.new_value, G_TYPE_INT);
+       g_value_set_int (&values.new_value, state);
+    }
+    else
+    {
+       g_value_init (&values.old_value, G_TYPE_INT);
+       g_value_set_int (&values.old_value, state);
+    }
+  g_signal_emit (accessible, atk_object_signals[PROPERTY_CHANGE],
+                 g_quark_from_string (atk_object_name_property_state),
+                 &values, NULL);
+}
+
 /**
  * atk_implementor_ref_accessible:
  * @implementor: The #GObject instance which should implement #AtkImplementorIface
@@ -793,7 +827,7 @@ atk_object_real_set_property (GObject      *object,
   accessible = ATK_OBJECT (object);
 
   switch (prop_id)
-  {
+    {
     case PROP_NAME:
       atk_object_set_name (accessible, g_value_get_string (value));
       break;
@@ -809,7 +843,7 @@ atk_object_real_set_property (GObject      *object,
       break;
     default:
       break;
-  }
+    }
 }
 
 static void
@@ -823,7 +857,7 @@ atk_object_real_get_property (GObject      *object,
   accessible = ATK_OBJECT (object);
 
   switch (prop_id)
-  {
+    {
     case PROP_NAME:
       g_value_set_string (value, atk_object_get_name (accessible));
       break;
@@ -836,7 +870,7 @@ atk_object_real_get_property (GObject      *object,
       break;
     default:
       break;
-  }
+    }
 }
 
 static void
@@ -897,23 +931,18 @@ atk_object_real_ref_state_set (AtkObject *accessible)
 
   ap = atk_object_get_parent (accessible);
   if (ap)
-  {
     if (ATK_IS_SELECTION (ap))
-    {
-      int i;
-
-      atk_state_set_add_state (state_set, ATK_STATE_SELECTABLE);
-
-      i = atk_object_get_index_in_parent (accessible);
-      if (i >= 0)
       {
-        if (atk_selection_is_child_selected(ATK_SELECTION (ap), i))
-        {
-          atk_state_set_add_state (state_set, ATK_STATE_SELECTED);
-        } 
+        int i;
+
+        atk_state_set_add_state (state_set, ATK_STATE_SELECTABLE);
+
+        i = atk_object_get_index_in_parent (accessible);
+        if (i >= 0)
+          if (atk_selection_is_child_selected(ATK_SELECTION (ap), i))
+            atk_state_set_add_state (state_set, ATK_STATE_SELECTED);
       } 
-    } 
-  }
+
   return state_set; 
 }
 
@@ -984,14 +1013,12 @@ static void
 atk_object_notify (GObject     *obj,
                    GParamSpec  *pspec)
 {
-  AtkPropertyValues values;
+  AtkPropertyValues values = { 0, };
 
-  memset (&values.old_value, 0, sizeof (GValue));
-  memset (&values.new_value, 0, sizeof (GValue));
   g_value_init (&values.new_value, pspec->value_type);
   g_object_get_property(obj, pspec->name, &values.new_value);
   values.property_name = pspec->name;
-  g_signal_emit (obj, atk_object_signals[PROPERTY_CHANGE], 0,
+  g_signal_emit (obj, atk_object_signals[PROPERTY_CHANGE],
+                 g_quark_from_string (pspec->name),
                  &values, NULL);
 }
-
