@@ -26,6 +26,68 @@ enum {
   LAST_SIGNAL
 };
 
+static const gchar * text_attr_name[] = {
+  "left_margin",
+  "right_margin",
+  "indent",
+  "invisible",
+  "editable",
+  "pixels_above_lines",
+  "pixels_below_lines",
+  "pixels_inside_wrap",
+  "bg_full_height",
+  "rise",
+  "underline",
+  "strikethrough",
+  "size",
+  "scale",
+  "weight",
+  "language",
+  "family_name",
+  "bg_color",
+  "fg_color",
+  "bg_stipple",
+  "fg_stipple",
+  "wrap_mode",
+  "direction",
+  "justification",
+  "stretch",
+  "variant",
+  "slant_style",
+};
+
+static const gchar *bool[] = {"false",
+                              "true"};
+static const gchar *style[] = {"normal",
+                               "oblique",
+                               "italic"};
+static const gchar *variant[] = {"normal",
+                                 "small_caps"};
+static const gchar *stretch[] = {"ultra_condensed",
+                                 "extra_condensed",
+                                 "condensed",
+                                 "semi_condensed",
+                                 "normal",
+                                 "semi_expanded",
+                                 "expanded",
+                                 "extra_expanded",
+                                 "ultra_expanded"};
+static const gchar *justification[] = {"left",
+                                       "right",
+                                       "center",
+
+                                       "fill"};
+static const gchar *direction[] = {"none",
+                                   "ltr",
+                                   "rtl"};
+static const gchar *wrap_mode[] = {"none",
+                                   "char",
+                                   "word"};
+static const gchar *underline[] = {"none",
+                                   "single",
+                                   "double",
+                                   "low"};
+
 struct _AtkTextIfaceClass
 {
   GObjectClass parent;
@@ -157,15 +219,44 @@ atk_text_get_character_at_offset (AtkText      *text,
  * @end_offset: the end offset of the returned string.
  *
  * Gets the specified text.
- * If the boundary type is ATK_TEXT_BOUNDARY_WORD_START or
- * ATK_TEXT_BOUNDARY_WORD_END part of a word may be returned.
- * If the boundary type is ATK_TEXT_BOUNDARY_SENTENCE_START the start point 
- * will be the offset and will continue to the start of the next sentence. 
- * The first word may not be a complete word. Similarly for 
- * ATK_TEXT_BOUNDARY_SENTENCE_END, ATK_TEXT_BOUNDARY_LINE_START and
- * ATK_TEXT_BOUNDARY_LINE_END
  *
- * Returns: the text after @offset up to the specified @boundary_type.
+ * If the boundary_type if ATK_TEXT_BOUNDARY_CHAR the character after the 
+ * offset is returned.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_WORD_START the returned string
+ * is from the word start after the offset to the next word start.
+ *
+ * The returned string will contain the word after the offset if the offset 
+ * is inside a word or if the offset is not inside a word.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_WORD_END the returned string
+ * is from the word end at or after the offset to the next work end.
+ *
+ * The returned string will contain the word after the offset if the offset
+ * is inside a word and will contain the word after the word after the offset
+ * if the offset is not inside a word.
+ *
+ * If the boundary type is ATK_TEXT_BOUNDARY_SENTENCE_START the returned
+ * string is from the sentence start after the offset to the next sentence
+ * start.
+ *
+ * The returned string will contain the sentence after the offset if the offset
+ * is inside a sentence or if the offset is not inside a sentence.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_SENTENCE_END the returned string
+ * is from the sentence end at or after the offset to the next sentence end.
+ *
+ * The returned string will contain the sentence after the offset if the offset
+ * is inside a sentence and will contain the sentence after the sentence
+ * after the offset if the offset is not inside a sentence.
+ *
+ * If the boundary type is ATK_TEXT_BOUNDARY_LINE_START the returned
+ * string is from the line start after the offset to the next line start.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_LINE_END the returned string
+ * is from the line end at or after the offset to the next line start.
+ *
+ * Returns: the text after @offset bounded by the specified @boundary_type.
  **/
 gchar*
 atk_text_get_text_after_offset (AtkText          *text,
@@ -206,14 +297,51 @@ atk_text_get_text_after_offset (AtkText          *text,
  * @end_offset: the end offset of the returned string.
  *
  * Gets the specified text.
- * If the boundary_type is ATK_TEXT_BOUNDARY_WORD_START or 
- * ATK_TEXT_BOUNDARY_WORD_END a complete word is returned; 
- * if the boundary type is  ATK_TEXT_BOUNDARY_SENTENCE_START or 
- * ATK_TEXT_BOUNDARY_SENTENCE_END a complete sentence
- * is returned; if the boundary type is ATK_TEXT_BOUNDARY_LINE_START or
- * ATK_TEXT_BOUNDARY_LINE_END a complete line is returned.
  *
- * Returns: the text at @offset up to the specified @boundary_type.
+ * If the boundary_type if ATK_TEXT_BOUNDARY_CHAR the character at the
+ * offset is returned.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_WORD_START the returned string
+ * is from the word start at or before the offset to the word start after 
+ * the offset.
+ *
+ * The returned string will contain the word at the offset if the offset
+ * is inside a word and will contain the word before the offset if the 
+ * offset is not inside a word.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_WORD_END the returned string
+ * is from the word end before the offset to the word end at or after the
+ * offset.
+ *
+ * The returned string will contain the word at the offset if the offset
+ * is inside a word and will contain the word after to the offset if the 
+ * offset is not inside a word.
+ *
+ * If the boundary type is ATK_TEXT_BOUNDARY_SENTENCE_START the returned
+ * string is from the sentence start at or before the offset to the sentence
+ * start after the offset.
+ *
+ * The returned string will contain the sentence at the offset if the offset
+ * is inside a sentence and will contain the sentence before the offset 
+ * if the offset is not inside a sentence.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_SENTENCE_END the returned string
+ * is from the sentence end before the offset to the sentence end at or
+ * after the offset.
+ *
+ * The returned string will contain the sentence at the offset if the offset
+ * is inside a sentence and will contain the sentence after the offset 
+ * if the offset is not inside a sentence.
+ *
+ * If the boundary type is ATK_TEXT_BOUNDARY_LINE_START the returned
+ * string is from the line start at or before the offset to the line
+ * start after the offset.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_LINE_END the returned string
+ * is from the line end before the offset to the line end at or after
+ * the offset.
+ *
+ * Returns: the text at @offset bounded by the specified @boundary_type.
  **/
 gchar*
 atk_text_get_text_at_offset (AtkText          *text,
@@ -254,15 +382,49 @@ atk_text_get_text_at_offset (AtkText          *text,
  * @end_offset: the end offset of the returned string.
  *
  * Gets the specified text.
- * If the boundary type is ATK_TEXT_BOUNDARY_WORD_START or
- * ATK_TEXT_BOUNDARY_WORD_END part of a word may be returned.
- * If the boundary type is ATK_TEXT_BOUNDARY_SENTENCE_START the start point 
- * will be at the start of the sentence, and will continue to the offset. 
- * The last word may not be a complete word. Similarly for 
- * ATK_TEXT_BOUNDARY_SENTENCE_END, ATK_TEXT_BOUNDARY_LINE_START and
- * ATK_TEXT_BOUNDARY_LINE_END
  *
- * Returns: the text before @offset starting from the specified @boundary_type.
+ * If the boundary_type if ATK_TEXT_BOUNDARY_CHAR the character before the
+ * offset is returned.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_WORD_START the returned string
+ * is from the word start before the word start before the offset to 
+ * the word start before the offset.
+ *
+ * The returned string will contain the word before the offset if the offset
+ * is inside a word and will contain the word before the word before the 
+ * offset if the offset is not inside a word.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_WORD_END the returned string
+ * is from the word end before the word end at or before the offset to the 
+ * word end at or before the offset.
+ *
+ * The returned string will contain the word before the offset if the offset
+ * is inside a word or if the offset is not inside a word.
+ *
+ * If the boundary type is ATK_TEXT_BOUNDARY_SENTENCE_START the returned
+ * string is from the sentence start before the sentence start before 
+ * the offset to the sentence start before the offset.
+ *
+ * The returned string will contain the sentence before the offset if the 
+ * offset is inside a sentence and will contain the sentence before the 
+ * sentence before the offset if the offset is not inside a sentence.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_SENTENCE_END the returned string
+ * is from the sentence end before the sentence end at or before the offset to 
+ * the sentence end at or before the offset.
+ *
+ * The returned string will contain the sentence before the offset if the 
+ * offset is inside a sentence or if the offset is not inside a sentence.
+ *
+ * If the boundary type is ATK_TEXT_BOUNDARY_LINE_START the returned
+ * string is from the line start before the line start ar or before the offset 
+ * to the line start ar or before the offset.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_LINE_END the returned string
+ * is from the line end before the line end before the offset to the 
+ * line end before the offset.
+ *
+ * Returns: the text before @offset bounded by the specified @boundary_type.
  **/
 gchar*
 atk_text_get_text_before_offset (AtkText          *text,
@@ -376,7 +538,7 @@ atk_text_get_character_extents (AtkText *text,
 }
 
 /**
- *atk_text_ref_run_attributes:
+ *atk_text_get_run_attributes:
  *@text: an #AtkText
  *@offset: the offset at which to get the attributes
  *@start_offset: the address to put the start offset of the range
@@ -385,17 +547,18 @@ atk_text_get_character_extents (AtkText *text,
  *Creates an #AtkAttributeSet which consists of the attributes explicitly
  *set at the position @offset in the text. @start_offset and @end_offset are
  *set to the start and end of the range around @offset where the attributes are
- *invariant. See the ATK_ATTRIBUTE macros, such as #ATK_ATTRIBUTE_LEFT_MARGIN
- *for types of text attributes that can be returned. Note that other 
- *attributes that do not have corresponding macros may also be returned.
+ *invariant. See the enum AtkTextAttribute for types of text attributes that 
+ *can be returned. Note that other attributes may also be returned.
  *
  *Returns: an #AtkAttributeSet which contains the attributes explicitly set
- *at @offset
+ *at @offset. This #AtkAttributeSet should be freed by a call to
+ *atk_attribute_set_free().
  **/
-AtkAttributeSet* atk_text_ref_run_attributes              (AtkText          *text,
-                                                           gint             offset,
-                                                           gint             *start_offset,
-                                                           gint             *end_offset)
+AtkAttributeSet* 
+atk_text_get_run_attributes (AtkText          *text,
+                             gint             offset,
+                             gint             *start_offset,
+                             gint             *end_offset)
 {
   AtkTextIface *iface;
   gint local_start_offset, local_end_offset;
@@ -410,12 +573,40 @@ AtkAttributeSet* atk_text_ref_run_attributes              (AtkText          *tex
   if (end_offset)
     real_end_offset = end_offset;
   else
-    real_start_offset = &local_end_offset;
+    real_end_offset = &local_end_offset;
 
   iface = ATK_TEXT_GET_IFACE (text);
 
-  if (iface->ref_run_attributes)
-    return (*(iface->ref_run_attributes)) (text, offset, real_start_offset, real_end_offset);
+  if (iface->get_run_attributes)
+    return (*(iface->get_run_attributes)) (text, offset, real_start_offset, real_end_offset);
+  else
+    return NULL;
+}
+
+/**
+ *atk_text_get_default_attributes:
+ *@text: an #AtkText
+ *
+ *Creates an #AtkAttributeSet which consists of the default values of
+ *attributes for the text. See the enum AtkTextAttribute for types of text 
+ *attributes that can be returned. Note that other attributes may also be 
+ *returned.
+ *
+ *Returns: an #AtkAttributeSet which contains the default values of attributes.
+ *at @offset. This #AtkAttributeSet should be freed by a call to
+ *atk_attribute_set_free().
+ */
+AtkAttributeSet* 
+atk_text_get_default_attributes (AtkText          *text)
+{
+  AtkTextIface *iface;
+
+  g_return_val_if_fail (ATK_IS_TEXT (text), NULL);
+
+  iface = ATK_TEXT_GET_IFACE (text);
+
+  if (iface->get_default_attributes)
+    return (*(iface->get_default_attributes)) (text);
   else
     return NULL;
 }
@@ -678,15 +869,85 @@ atk_text_set_caret_offset (AtkText *text,
 void
 atk_attribute_set_free(AtkAttributeSet *attrib_set)
 {
-  gint index;
+  GSList *temp;
 
-  if (attrib_set == NULL)
-    return;
+  temp = attrib_set;
 
-  for (index = 0; index < g_slist_length(attrib_set); index++)
-  {
-    g_free(((AtkAttribute*) (g_slist_nth(attrib_set,index)->data))->name);
-    g_free(((AtkAttribute*) (g_slist_nth(attrib_set,index)->data))->value);
-  }
-  g_slist_free(attrib_set);
+  while (temp != NULL)
+    {
+      AtkAttribute *att;
+
+      att = temp->data;
+
+      g_free (att->name);
+      g_free (att->value);
+      g_free (att);
+      temp = temp->next;
+    }
+  g_slist_free (attrib_set);
+}
+
+/**
+ * atk_attribute_get_name:
+ * @attr: The #AtkTextAttribute whose name is required
+ *
+ * Gets the name corresponding to the #AtkTextAttribute
+ *
+ * Returns: a string containing the name; this string should not be freed
+ **/
+G_CONST_RETURN gchar*
+atk_attribute_get_name (AtkTextAttribute attr)
+{
+  g_assert (attr >= 0 && attr <= ATK_TEXT_ATTR_STYLE);
+  return text_attr_name[attr];
+}
+
+/**
+ * atk_attribute_get_value:
+ * @attr: The #AtkTextAttribute for which a value is required
+ * @index: The index of the required value
+ *
+ * Gets the value for the index of the #AtkTextAttribute
+ *
+ * Returns: a string containing the value; this string should not be freed;
+ * NULL is returned if there are no values maintained for the attr value. 
+ **/
+G_CONST_RETURN gchar*
+atk_attribute_get_value (AtkTextAttribute attr,
+                         gint             index)
+{
+  switch (attr)
+    {
+    case ATK_TEXT_ATTR_INVISIBLE:
+    case ATK_TEXT_ATTR_EDITABLE:
+    case ATK_TEXT_ATTR_BG_FULL_HEIGHT:
+    case ATK_TEXT_ATTR_STRIKETHROUGH:
+    case ATK_TEXT_ATTR_BG_STIPPLE:
+    case ATK_TEXT_ATTR_FG_STIPPLE:
+      g_assert (index >= 0 && index < 2);
+      return bool[index];
+    case ATK_TEXT_ATTR_UNDERLINE:
+      g_assert (index >= 0 && index < 4);
+      return underline[index];
+    case ATK_TEXT_ATTR_WRAP_MODE:
+      g_assert (index >= 0 && index < 3);
+      return wrap_mode[index];
+    case ATK_TEXT_ATTR_DIRECTION:
+      g_assert (index >= 0 && index < 3);
+      return direction[index];
+    case ATK_TEXT_ATTR_JUSTIFICATION:
+      g_assert (index >= 0 && index < 3);
+      return justification[index];
+    case ATK_TEXT_ATTR_STRETCH:
+      g_assert (index >= 0 && index < 9);
+      return stretch[index];
+    case ATK_TEXT_ATTR_VARIANT:
+      g_assert (index >= 0 && index < 2);
+      return variant[index];
+    case ATK_TEXT_ATTR_STYLE:
+      g_assert (index >= 0 && index < 3);
+      return style[index];
+    default:
+      return NULL;
+   }
 }
